Package: sps
Title: Sequential Poisson Sampling
Version: 0.5.4
Authors@R: c(
    person("Steve", "Martin",
            role = c("aut", "cre", "cph"),
            email = "marberts@protonmail.com",
            comment = c(ORCID = "0000-0003-2544-9480")),
    person("Justin", "Francis", role = "ctb")
    )
Description: Sequential Poisson sampling is a variation of Poisson sampling for
    drawing probability-proportional-to-size samples with a given number of
    units, and is commonly used for price-index surveys. This package gives
    functions to draw stratified sequential Poisson samples according to the
    method by Ohlsson (1998, ISSN:0282-423X), as well as other order sample
    designs by Rosén (1997, <doi:10.1016/S0378-3758(96)00186-3>), and generate
    appropriate bootstrap replicate weights according to the generalized
    bootstrap method by Beaumont and Patak
    (2012, <doi:10.1111/j.1751-5823.2011.00166.x>).
Depends: R (>= 4.0)
Imports: stats
Suggests: knitr, rmarkdown, testthat (>= 3.0.0)
Config/testthat/edition: 3
License: MIT + file LICENSE
Encoding: UTF-8
URL: https://marberts.github.io/sps/, https://github.com/marberts/sps
BugReports: https://github.com/marberts/sps/issues
VignetteBuilder: knitr
Roxygen: list(markdown = TRUE)
RoxygenNote: 7.3.1
Author: Steve Martin [aut, cre, cph] (<https://orcid.org/0000-0003-2544-9480>),
  Justin Francis [ctb]
Maintainer: Steve Martin <marberts@protonmail.com>
Built: R 4.4.0; ; 2024-05-03 04:34:38 UTC; unix
